@@ -386,10 +386,9 @@ class ActionExecutor:
             tracker = Tracker.from_dict(tracker_json)
             dispatcher = CollectingDispatcher()
 
-            if utils.is_coroutine_action(action):
-                events = await action(dispatcher, tracker, domain)
-            else:
-                events = action(dispatcher, tracker, domain)
+            events = await utils.call_potential_coroutine(
+                action(dispatcher, tracker, domain)
+            )
 
             if not events:
                 # make sure the action did not just return `None`...
